// IPCamera.cpp : Defines the entry point for the application.
//

#include <stdio.h>
#include "LuxandFaceSDK.h"
#include "LuxandFaceSDK_RTSP.h"

HDC dc = NULL;
RECT ClientRect;
HTracker tracker = 0;

volatile int stop = 0;
volatile int is_running = 0;

void processFrame(unsigned char * data, int width, int height, int scanline);

int cameraHandle;

int main(int argc, char* argv[])
{
	if (FSDKE_OK != FSDK_ActivateLibrary("")){
		MessageBox(0, L"Please run the License Key Wizard (Start - Luxand - FaceSDK - License Key Wizard)\n", L"Error activating FaceSDK", MB_ICONERROR | MB_OK);
        exit(-1);
    }

	FSDK_Initialize("");
	FSDK_RTSP_Init();

	// creating a window
	HWND hwnd = CreateWindowEx(WS_EX_TOOLWINDOW, L"LISTBOX", L"IP Camera Face Tracking", 0, 0, 0, 0, 0, 0, 0, 0, 0); 
	dc = GetDC(hwnd);
	SetWindowPos(hwnd, 0, 0, 0, 660, 540, SWP_NOZORDER|SWP_NOMOVE);
	ShowWindow(hwnd, SW_SHOW);
	GetClientRect(hwnd, &ClientRect);
	HPEN hNPen = CreatePen(PS_SOLID, 1, RGB(0, 255, 0));
	HPEN hOPen = (HPEN)SelectObject(dc, hNPen);
	HBRUSH hOldBrush;
	HBRUSH hNewBrush;
	hNewBrush = (HBRUSH)GetStockObject(NULL_BRUSH);
	hOldBrush = (HBRUSH)SelectObject(dc, hNewBrush);

	HFONT myFont = CreateFont(14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, L"Microsoft Sans Serif");

	// creating input boxes 
	HWND Address = CreateWindow(L"STATIC", L"address:", WS_CHILD, 0, 0, 0, 0, hwnd, 0, 0, 0); 
	SendMessage(Address, WM_SETFONT, (WPARAM)myFont, TRUE);
	SetWindowPos(Address, 0, 5, ClientRect.bottom - 20, 40, 18, SWP_NOZORDER);
	ShowWindow(Address, SW_SHOW);

	HWND AddressBox = CreateWindow(L"EDIT", L"rtsp://192.168.1.6:554", WS_CHILD | WS_BORDER | ES_AUTOHSCROLL, 0, 0, 0, 0, hwnd, 0, 0, 0); 

	SendMessage(AddressBox, WM_SETFONT, (WPARAM)myFont, TRUE);
	SetWindowPos(AddressBox, 0, 45, ClientRect.bottom - 20, 500/*360*/, 18, SWP_NOZORDER);
	ShowWindow(AddressBox, SW_SHOW);

	SendMessage(hwnd, LB_ADDSTRING, 0, (LPARAM)(L"Press Esc to exit ..."));

	// creating a Tracker
	FSDK_CreateTracker(&tracker);

	int err = 0;	
	FSDK_SetTrackerMultipleParameters(tracker, "RecognizeFaces=false; DetectFacialFeatures=true; HandleArbitraryRotations=false; DetermineFaceRotationAngle=false; InternalResizeWidth=256; FaceDetectionThreshold=5;", &err);
						
	int cameraHandle = 0;
	bool CameraOpened = false;
	bool CameraReopened = false;

	char Camera_URL[1024];
	MSG msg = {0};
	while (msg.message != WM_QUIT) {
		if (CameraOpened) {
			unsigned char * data;
			int width, height, scanline;
			int res = FSDK_RTSP_GrabFrame(cameraHandle, &data, &width, &height, &scanline);
			if (res) {
				if (CameraReopened) {
					FSDK_RTSP_CloseCamera(cameraHandle, 10);
				}
				CameraReopened = false;
				printf("Trying to re-open the camera...\n");
				res = FSDK_RTSP_OpenCamera(Camera_URL, 10, &cameraHandle);
				if (!res) {
					printf("Camera opened.\n");
					CameraReopened = true;
				}
			} else {
				processFrame(data, width, height, scanline);
			}
		}
		if (PeekMessage(&msg, 0, 0, 0, PM_REMOVE)) {
			TranslateMessage(&msg);   
			DispatchMessage(&msg); 
			if (msg.message == WM_KEYDOWN) {
				if (msg.wParam == VK_ESCAPE) 
					break;
				if (msg.wParam == VK_RETURN) {
					SendMessageA(AddressBox, WM_GETTEXT, (WPARAM) 1024, (LPARAM) Camera_URL);
	
					if (CameraOpened) {
						FSDK_RTSP_CloseCamera(cameraHandle, 10);
					}
					printf("Trying to open the camera...\n");
					if (!FSDK_RTSP_OpenCamera(Camera_URL, 10, &cameraHandle)) {
						printf("Camera opened.\n");
						CameraOpened = true;
						CameraReopened = true;
					} else {
						printf("Error opening camera.\n");
						CameraOpened = false;
					}
				}
			}
		}
		Sleep(20);
	}

	ReleaseDC(hwnd, dc);

	return 0;
}

void processFrame(unsigned char * frameData, int frameWidth, int frameHeight, int frameScanline) {
	HImage imageHandle;
	int res = FSDK_LoadImageFromBuffer(&imageHandle, frameData, frameWidth, frameHeight, frameScanline, FSDK_IMAGE_COLOR_24BIT);
	if (res != FSDKE_OK) {
		printf("Cannot read image\n");
		return;
	}

	long long IDs[256];
	long long faceCount = 0;
	FSDK_FeedFrame(tracker, 0, imageHandle, &faceCount, IDs, sizeof(IDs));

	HImage resizedImageHandle;
	FSDK_CreateEmptyImage(&resizedImageHandle);

	int width;
	int height;
	FSDK_GetImageWidth(imageHandle, &width);
	FSDK_GetImageHeight(imageHandle, &height);

	float ratio = min(ClientRect.right / (float)width, (ClientRect.bottom - 40) / (float)height);
	FSDK_ResizeImage(imageHandle, ratio, resizedImageHandle);
	FSDK_FreeImage(imageHandle);

	FSDK_GetImageWidth(resizedImageHandle, &width);
	FSDK_GetImageWidth(resizedImageHandle, &height);

	HBITMAP hbitmapHandle; // to store the HBITMAP handle
	FSDK_SaveImageToHBitmap(resizedImageHandle, &hbitmapHandle);

	DrawState(dc, NULL, NULL, (LPARAM)hbitmapHandle, NULL, 0, 16, width, height, DST_BITMAP | DSS_NORMAL);

	for (int i = 0; i < faceCount; ++i) {
		TFacePosition facePosition;
		FSDK_GetTrackerFacePosition(tracker, 0, IDs[i], &facePosition);

		int x1 = (int)(ratio*(facePosition.xc - facePosition.w*0.6));
		int y1 = (int)(ratio*(facePosition.yc - facePosition.w*0.5));
		int x2 = (int)(ratio*(facePosition.xc + facePosition.w*0.6));
		int y2 = (int)(ratio*(facePosition.yc + facePosition.w*0.7));
		Rectangle(dc, x1, 16 + y1, x2, 16 + y2);
	}

	DeleteObject(hbitmapHandle); // delete the HBITMAP object
	FSDK_FreeImage(resizedImageHandle);// delete the FSDK image handle
}
